@@ -60,16 +60,10 @@ void dump_stack(struct thread *thread)
     unsigned long *bottom = (unsigned long *)(thread->stack + STACK_SIZE); 
     unsigned long *pointer = (unsigned long *)thread->sp;
     int count;
-    if(thread == current)
-    {
-#ifdef __i386__    
-        asm("movl %%esp,%0"
-            : "=r"(pointer));
-#else
-        asm("movq %%rsp,%0"
-            : "=r"(pointer));
-#endif
-    }
+
+    if ( thread == current )
+        asm ( "mov %%"ASM_SP",%0" : "=r" (pointer) );
+
     printk("The stack for \"%s\"\n", thread->name);
     for(count = 0; count < 25 && pointer < bottom; count ++)
     {
@@ -119,21 +113,46 @@ struct thread* arch_create_thread(char *name, void (*function)(void *),
 
 void run_idle_thread(void)
 {
-    /* Switch stacks and run the thread */ 
-#if defined(__i386__)
-    __asm__ __volatile__("mov %0,%%esp\n\t"
-                         "push %1\n\t" 
-                         "ret"                                            
-                         :"=m" (idle_thread->sp)
-                         :"m" (idle_thread->ip));                          
-#elif defined(__x86_64__)
-    __asm__ __volatile__("mov %0,%%rsp\n\t"
-                         "push %1\n\t" 
-                         "ret"                                            
-                         :"=m" (idle_thread->sp)
-                         :"m" (idle_thread->ip));                                                    
-#endif
+    /* Switch stacks and run the thread */
+    asm volatile ( "mov %[sp], %%"ASM_SP"\n\t"
+                   "jmp *%[ip]\n\t"
+                   :
+                   : [sp] "m" (idle_thread->sp),
+                     [ip] "m" (idle_thread->ip) );
 }
 
+unsigned long __local_irq_save(void)
+{
+    unsigned long flags;
 
+    local_irq_save(flags);
+    return flags;
+}
+EXPORT_SYMBOL(__local_irq_save);
 
+void __local_irq_restore(unsigned long flags)
+{
+    local_irq_restore(flags);
+}
+EXPORT_SYMBOL(__local_irq_restore);
+
+unsigned long __local_save_flags(void)
+{
+    unsigned long flags;
+
+    local_save_flags(flags);
+    return flags;
+}
+EXPORT_SYMBOL(__local_save_flags);
+
+void __local_irq_disable(void)
+{
+    local_irq_disable();
+}
+EXPORT_SYMBOL(__local_irq_disable);
+
+void __local_irq_enable(void)
+{
+    local_irq_enable();
+}
+EXPORT_SYMBOL(__local_irq_enable);
